package java.io;

import java.util.Locale;
import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.Charset;
import java.util.Formatter;

public class PrintStream extends FilterOutputStream implements Appendable, Closeable
{
    private final boolean autoFlush;
    private boolean trouble;
    private Formatter formatter;
    private BufferedWriter textOut;
    private OutputStreamWriter charOut;
    private boolean closing;
    
    private static <T> T requireNonNull(T t, String s) {
        if (t == null) {
            throw new NullPointerException(s);
        }
        return t;
    }
    
    private static Charset toCharset(String charsetName) throws UnsupportedEncodingException {
        requireNonNull(charsetName, "charsetName");
        try {
            return Charset.forName(charsetName);
        }
        catch (IllegalCharsetNameException | UnsupportedCharsetException ex) {
            throw new UnsupportedEncodingException(charsetName);
        }
    }
    
    private PrintStream(boolean autoFlush, OutputStream outputStream) {
        super(outputStream);
        this.trouble = false;
        this.closing = false;
        this.autoFlush = autoFlush;
        this.charOut = new OutputStreamWriter(this);
        this.textOut = new BufferedWriter(this.charOut);
    }
    
    private PrintStream(boolean autoFlush, OutputStream outputStream, Charset charset) {
        super(outputStream);
        this.trouble = false;
        this.closing = false;
        this.autoFlush = autoFlush;
        this.charOut = new OutputStreamWriter(this, charset);
        this.textOut = new BufferedWriter(this.charOut);
    }
    
    private PrintStream(boolean b, Charset charset, OutputStream outputStream) throws UnsupportedEncodingException {
        this(b, outputStream, charset);
    }
    
    public PrintStream(OutputStream outputStream) {
        this(outputStream, false);
    }
    
    public PrintStream(OutputStream outputStream, boolean b) {
        this(b, (OutputStream)requireNonNull(outputStream, "Null output stream"));
    }
    
    public PrintStream(OutputStream outputStream, boolean b, String s) throws UnsupportedEncodingException {
        this(b, (OutputStream)requireNonNull(outputStream, "Null output stream"), toCharset(s));
    }
    
    public PrintStream(String s) throws FileNotFoundException {
        this(false, new FileOutputStream(s));
    }
    
    public PrintStream(String s, String s2) throws FileNotFoundException, UnsupportedEncodingException {
        this(false, toCharset(s2), new FileOutputStream(s));
    }
    
    public PrintStream(File file) throws FileNotFoundException {
        this(false, new FileOutputStream(file));
    }
    
    public PrintStream(File file, String s) throws FileNotFoundException, UnsupportedEncodingException {
        this(false, toCharset(s), new FileOutputStream(file));
    }
    
    private void ensureOpen() throws IOException {
        if (this.out == null) {
            throw new IOException("Stream closed");
        }
        return;
    }
    
    @Override
    public void flush() {
        synchronized (this) {
            try {
                this.ensureOpen();
                this.out.flush();
            }
            catch (IOException ex) {
                this.trouble = true;
            }
        }
    }
    
    @Override
    public void close() {
        synchronized (this) {
            if (!this.closing) {
                this.closing = true;
                try {
                    this.textOut.close();
                    this.out.close();
                }
                catch (IOException ex) {
                    this.trouble = true;
                }
                this.textOut = null;
                this.charOut = null;
                this.out = null;
            }
        }
    }
    
    public boolean checkError() {
        if (this.out != null) {
            this.flush();
        }
        if ((this.out instanceof PrintStream)) {
            return ((PrintStream)this.out).checkError();
        }
        return this.trouble;
    }
    
    protected void setError() {
        this.trouble = true;
    }
    
    protected void clearError() {
        this.trouble = false;
    }
    
    @Override
    public void write(int n) {
        try {
            synchronized (this) {
                this.ensureOpen();
                this.out.write(n);
                if (n == 10 && this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    @Override
    public void write(byte[] array, int n, int n2) {
        try {
            synchronized (this) {
                this.ensureOpen();
                this.out.write(array, n, n2);
                if (this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    private void write(char[] array) {
        try {
            synchronized (this) {
                this.ensureOpen();
                this.textOut.write(array);
                this.textOut.flushBuffer();
                this.charOut.flushBuffer();
                if (this.autoFlush) {
                    for (int i = 0; i < array.length; ++i) {
                        if (array[i] == '\n') {
                            this.out.flush();
                        }
                    }
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    private void write(String s) {
        try {
            synchronized (this) {
                this.ensureOpen();
                this.textOut.write(s);
                this.textOut.flushBuffer();
                this.charOut.flushBuffer();
                if (this.autoFlush && s.indexOf(10) >= 0) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    private void newLine() {
        try {
            synchronized (this) {
                this.ensureOpen();
                this.textOut.newLine();
                this.textOut.flushBuffer();
                this.charOut.flushBuffer();
                if (this.autoFlush) {
                    this.out.flush();
                }
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
    }
    
    public void print(boolean b) {
        this.write(b ? "true" : "false");
    }
    
    public void print(char c) {
        this.write(String.valueOf(c));
    }
    
    public void print(int i) {
        this.write(String.valueOf(i));
    }
    
    public void print(long l) {
        this.write(String.valueOf(l));
    }
    
    public void print(float f) {
        this.write(String.valueOf(f));
    }
    
    public void print(double d) {
        this.write(String.valueOf(d));
    }
    
    public void print(char[] array) {
        this.write(array);
    }
    
    public void print(String s) {
        if (s == null) {
            s = "null";
        }
        this.write(s);
    }
    
    public void print(Object obj) {
        this.write(String.valueOf(obj));
    }
    
    public void println() {
        this.newLine();
    }
    
    public void println(boolean b) {
        synchronized (this) {
            this.print(b);
            this.newLine();
        }
    }
    
    public void println(char c) {
        synchronized (this) {
            this.print(c);
            this.newLine();
        }
    }
    
    public void println(int n) {
        synchronized (this) {
            this.print(n);
            this.newLine();
        }
    }
    
    public void println(long n) {
        synchronized (this) {
            this.print(n);
            this.newLine();
        }
    }
    
    public void println(float n) {
        synchronized (this) {
            this.print(n);
            this.newLine();
        }
    }
    
    public void println(double n) {
        synchronized (this) {
            this.print(n);
            this.newLine();
        }
    }
    
    public void println(char[] array) {
        synchronized (this) {
            this.print(array);
            this.newLine();
        }
    }
    
    public void println(String s) {
        synchronized (this) {
            this.print(s);
            this.newLine();
        }
    }
    
    public void println(Object obj) {
        String value = String.valueOf(obj);
        synchronized (this) {
            this.print(value);
            this.newLine();
        }
    }
    
    public PrintStream printf(String s, Object... array) {
        return this.format(s, array);
    }
    
    public PrintStream printf(Locale locale, String s, Object... array) {
        return this.format(locale, s, array);
    }
    
    public PrintStream format(String format, Object... args) {
        try {
            synchronized (this) {
                this.ensureOpen();
                if (this.formatter == null || this.formatter.locale() != Locale.getDefault()) {
                    this.formatter = new Formatter(this);
                }
                this.formatter.format(Locale.getDefault(), format, args);
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
        return this;
    }
    
    public PrintStream format(Locale locale, String format, Object... args) {
        try {
            synchronized (this) {
                this.ensureOpen();
                if (this.formatter == null || this.formatter.locale() != locale) {
                    this.formatter = new Formatter(this, locale);
                }
                this.formatter.format(locale, format, args);
            }
        }
        catch (InterruptedIOException ex) {
            Thread.currentThread().interrupt();
        }
        catch (IOException ex2) {
            this.trouble = true;
        }
        return this;
    }
    
    @Override
    public PrintStream append(CharSequence charSequence) {
        if (charSequence == null) {
            this.print("null");
            return this;
        }
        this.print(charSequence.toString());
        return this;
    }
    
    @Override
    public PrintStream append(CharSequence charSequence, int n, int n2) {
        charSequence = ((charSequence == null) ? "null" : charSequence);
        this.write(charSequence.subSequence(n, n2).toString());
        return this;
    }
    
    @Override
    public PrintStream append(char c) {
        this.print(c);
        return this;
    }
}
